@@ -1,5 +1,5 @@
 /*
- * $Id: Section.java 5951 2013-08-09 12:13:00Z achingarev $
+ * $Id: Section.java 6074 2013-11-19 09:58:52Z achingarev $
  *
  * This file is part of the iText (R) project.
  * Copyright (c) 1998-2013 1T3XT BVBA
@@ -173,7 +173,8 @@ public class Section extends ArrayList<Element> implements TextElementArray, Lar
     protected Section(final Paragraph title, final int numberDepth) {
         this.numberDepth = numberDepth;
         this.title = title;
-        title.setRole(new PdfName("H" + numberDepth));
+        if (title != null)
+            title.setRole(new PdfName("H" + numberDepth));
     }
 
     // implementation of the Element-methods
