@@ -1,5 +1,5 @@
 /*
- * $Id: BidiLine.java 5914 2013-07-28 14:18:11Z blowagie $
+ * $Id: BidiLine.java 6015 2013-09-17 13:48:01Z eugenemark $
  *
  * This file is part of the iText (R) project.
  * Copyright (c) 1998-2013 1T3XT BVBA
@@ -359,9 +359,7 @@ public class BidiLine {
             if (ck.isImage() && minY < yLine) {
                 Image img = ck.getImage();
                 if (img.isScaleToFitHeight() && yLine + 2 * descender - img.getScaledHeight() - ck.getImageOffsetY() - img.getSpacingBefore() < minY) {
-                    //float scalePercent = (yLine + 2 * descender - ck.getImageOffsetY() - img.getSpacingBefore() - minY) / img.getHeight() * 100;
-            		//img.scalePercent(scalePercent);
-                    float scalePercent = (yLine + 2 * descender - ck.getImageOffsetY() - img.getSpacingBefore() - minY) / img.getHeight();
+                    float scalePercent = (yLine + 2 * descender - ck.getImageOffsetY() - img.getSpacingBefore() - minY) / img.getScaledHeight();
                 	ck.setImageScalePercentage(scalePercent);
                 }
             }
