@@ -1,5 +1,5 @@
 /*
- * $Id: PdfChunk.java 5914 2013-07-28 14:18:11Z blowagie $
+ * $Id: PdfChunk.java 6086 2013-11-26 11:19:45Z pavel-alay $
  *
  * This file is part of the iText (R) project.
  * Copyright (c) 1998-2013 1T3XT BVBA
@@ -150,6 +150,8 @@ public class PdfChunk {
 
     protected IAccessibleElement accessibleElement = null;
 
+    public static final float UNDERLINE_THICKNESS = 1f / 15;
+    public static final float UNDERLINE_OFFSET = -1f / 3;
     // constructors
 
 /**
@@ -234,7 +236,7 @@ public class PdfChunk {
             }
         }
         if (f.isUnderlined()) {
-            Object obj[] = {null, new float[]{0, 1f / 15, 0, -1f / 3, 0}};
+            Object obj[] = {null, new float[]{0, UNDERLINE_THICKNESS, 0, UNDERLINE_OFFSET, 0}};
             Object unders[][] = Utilities.addToArray((Object[][])attributes.get(Chunk.UNDERLINE), obj);
             attributes.put(Chunk.UNDERLINE, unders);
         }
