@@ -1,5 +1,5 @@
 /*
- * $Id: PdfEncryption.java 5994 2013-09-08 10:25:34Z blowagie $
+ * $Id: PdfEncryption.java 6045 2013-10-29 11:39:04Z blowagie $
  *
  * This file is part of the iText (R) project.
  * Copyright (c) 1998-2013 1T3XT BVBA
@@ -551,7 +551,7 @@ public class PdfEncryption {
 			keySize = 16;
 	}
 
-	public static PdfObject createInfoId(byte id[]) throws IOException {
+	public static PdfObject createInfoId(byte id[], boolean modified) throws IOException {
 		ByteBuffer buf = new ByteBuffer(90);
 		buf.append('[').append('<');
 		if (id.length != 16)
@@ -559,7 +559,8 @@ public class PdfEncryption {
 		for (int k = 0; k < 16; ++k)
 			buf.appendHex(id[k]);
 		buf.append('>').append('<');
-		id = createDocumentId();
+		if (modified)
+			id = createDocumentId();
 		for (int k = 0; k < 16; ++k)
 			buf.appendHex(id[k]);
 		buf.append('>').append(']');
@@ -733,8 +734,8 @@ public class PdfEncryption {
 		return dic;
 	}
 
-	public PdfObject getFileID() throws IOException {
-		return createInfoId(documentID);
+	public PdfObject getFileID(boolean modified) throws IOException {
+		return createInfoId(documentID, modified);
 	}
 
 	public OutputStreamEncryption getEncryptionStream(OutputStream os) {
