@@ -1,5 +1,5 @@
 /*
- * $Id: PdfPRow.java 5951 2013-08-09 12:13:00Z achingarev $
+ * $Id: PdfPRow.java 6020 2013-09-25 10:47:31Z blowagie $
  *
  * This file is part of the iText (R) project.
  * Copyright (c) 1998-2013 1T3XT BVBA
@@ -44,7 +44,6 @@
 package com.itextpdf.text.pdf;
 
 import java.util.HashMap;
-import java.util.UUID;
 
 import com.itextpdf.text.*;
 import com.itextpdf.text.log.Logger;
@@ -84,6 +83,7 @@ public class PdfPRow implements IAccessibleElement {
 	protected float maxHeight = 0;
 	
 	protected boolean calculated = false;
+	protected boolean adjusted = false;
     
     private int[] canvasesPos;
 
@@ -845,7 +845,15 @@ public class PdfPRow implements IAccessibleElement {
 		return false;
 	}
 
-    public PdfObject getAccessibleAttribute(final PdfName key) {
+    public boolean isAdjusted() {
+		return adjusted;
+	}
+
+	public void setAdjusted(boolean adjusted) {
+		this.adjusted = adjusted;
+	}
+
+	public PdfObject getAccessibleAttribute(final PdfName key) {
         if (accessibleAttributes != null)
             return accessibleAttributes.get(key);
         else
