@@ -1,5 +1,5 @@
 /*
- * $Id: PdfStamper.java 5934 2013-08-06 15:21:45Z eugenemark $
+ * $Id: PdfStamper.java 6103 2013-11-28 08:41:24Z blowagie $
  *
  * This file is part of the iText (R) project.
  * Copyright (c) 1998-2013 1T3XT BVBA
@@ -572,10 +572,11 @@ public class PdfStamper
      * Sets the document's compression to the new 1.5 mode with object streams and xref
      * streams. It can be set at any time but once set it can't be unset.
      */
-    public void setFullCompression() {
+    public void setFullCompression() throws DocumentException {
         if (stamper.isAppend())
             return;
-        stamper.setFullCompression();
+        stamper.fullCompression = true;
+        stamper.setAtLeastPdfVersion(PdfWriter.VERSION_1_5);
     }
 
     /**
