@@ -1,5 +1,5 @@
 /*
- * $Id: PdfStamperImp.java 5985 2013-09-05 07:29:27Z blowagie $
+ * $Id: PdfStamperImp.java 6045 2013-10-29 11:39:04Z blowagie $
  *
  * This file is part of the iText (R) project.
  * Copyright (c) 1998-2013 1T3XT BVBA
@@ -156,6 +156,30 @@ class PdfStamperImp extends PdfWriter {
                 fullCompression = false;
         }
         initialXrefSize = reader.getXrefSize();
+        readColorProfile();
+    }
+
+    protected void readColorProfile() {
+        PdfObject outputIntents = reader.getCatalog().getAsArray(PdfName.OUTPUTINTENTS);
+        if (outputIntents != null && ((PdfArray) outputIntents).size() > 0) {
+            PdfStream iccProfileStream = null;
+            for (int i = 0; i < ((PdfArray) outputIntents).size(); i++) {
+                PdfDictionary outputIntentDictionary = ((PdfArray) outputIntents).getAsDict(i);
+                if (outputIntentDictionary != null) {
+                    iccProfileStream = outputIntentDictionary.getAsStream(PdfName.DESTOUTPUTPROFILE);
+                    if (iccProfileStream != null)
+                        break;
+                }
+            }
+
+            if (iccProfileStream instanceof PRStream) {
+                try {
+                    colorProfile = ICC_Profile.getInstance(PdfReader.getStreamBytes((PRStream)iccProfileStream));
+                } catch(IOException exc) {
+                    throw new ExceptionConverter(exc);
+                }
+            }
+        }
     }
 
     protected void setViewerPreferences() {
@@ -194,6 +218,10 @@ class PdfStamperImp extends PdfWriter {
         setOutlines();
         setJavaScript();
         addFileAttachments();
+        // [C11] Output Intents
+        if (extraCatalog != null) {
+            catalog.mergeDifferent(extraCatalog);
+        }
         if (openAction != null) {
             catalog.put(PdfName.OPENACTION, openAction);
         }
@@ -382,15 +410,15 @@ class PdfStamperImp extends PdfWriter {
                 PdfIndirectObject encryptionObject = addToBody(crypto.getEncryptionDictionary(), false);
                 encryption = encryptionObject.getIndirectReference();
             }
-            fileID = crypto.getFileID();
+            fileID = crypto.getFileID(true);
         }
         else {
         	PdfArray IDs = reader.trailer.getAsArray(PdfName.ID);
         	if (IDs != null && IDs.getAsString(0) != null) {
-                fileID = PdfEncryption.createInfoId(IDs.getAsString(0).getBytes());
+                fileID = PdfEncryption.createInfoId(IDs.getAsString(0).getBytes(), true);
         	}
         	else {
-                fileID = PdfEncryption.createInfoId(PdfEncryption.createDocumentId());
+                fileID = PdfEncryption.createInfoId(PdfEncryption.createDocumentId(), true);
         	}	
         }
         PRIndirectReference iRoot = (PRIndirectReference)reader.trailer.get(PdfName.ROOT);
