@@ -1,5 +1,5 @@
 /*
- * $Id: MakeXmlSignature.java 6001 2013-09-09 13:17:37Z pavel-alay $
+ * $Id: MakeXmlSignature.java 6033 2013-10-16 09:28:50Z pavel-alay $
  *
  * This file is part of the iText (R) project.
  * Copyright (c) 1998-2013 1T3XT BVBA
@@ -46,11 +46,7 @@ package com.itextpdf.text.pdf.security;
 import com.itextpdf.text.DocumentException;
 import com.itextpdf.text.error_messages.MessageLocalization;
 import com.itextpdf.text.pdf.XmlSignatureAppearance;
-import org.apache.jcp.xml.dsig.internal.dom.DOMKeyInfoFactory;
-import org.apache.jcp.xml.dsig.internal.dom.DOMReference;
-import org.apache.jcp.xml.dsig.internal.dom.DOMSignedInfo;
-import org.apache.jcp.xml.dsig.internal.dom.DOMXMLSignature;
-import org.apache.jcp.xml.dsig.internal.dom.DOMUtils;
+import org.apache.jcp.xml.dsig.internal.dom.*;
 import org.apache.xml.security.utils.Base64;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
@@ -59,13 +55,7 @@ import org.w3c.dom.NodeList;
 
 import javax.xml.crypto.XMLStructure;
 import javax.xml.crypto.dom.DOMStructure;
-import javax.xml.crypto.dsig.CanonicalizationMethod;
-import javax.xml.crypto.dsig.Reference;
-import javax.xml.crypto.dsig.SignatureMethod;
-import javax.xml.crypto.dsig.XMLObject;
-import javax.xml.crypto.dsig.XMLSignatureFactory;
-import javax.xml.crypto.dsig.DigestMethod;
-import javax.xml.crypto.dsig.Transform;
+import javax.xml.crypto.dsig.*;
 import javax.xml.crypto.dsig.dom.DOMSignContext;
 import javax.xml.crypto.dsig.keyinfo.KeyInfo;
 import javax.xml.crypto.dsig.keyinfo.KeyInfoFactory;
@@ -75,8 +65,12 @@ import javax.xml.crypto.dsig.spec.C14NMethodParameterSpec;
 import javax.xml.crypto.dsig.spec.TransformParameterSpec;
 import javax.xml.crypto.dsig.spec.XPathFilter2ParameterSpec;
 import javax.xml.crypto.dsig.spec.XPathType;
-import java.io.ByteArrayOutputStream;
-import java.io.IOException;
+import javax.xml.transform.OutputKeys;
+import javax.xml.transform.Transformer;
+import javax.xml.transform.TransformerFactory;
+import javax.xml.transform.dom.DOMSource;
+import javax.xml.transform.stream.StreamResult;
+import java.io.*;
 import java.security.GeneralSecurityException;
 import java.security.Key;
 import java.security.MessageDigest;
@@ -84,11 +78,7 @@ import java.security.PublicKey;
 import java.security.cert.Certificate;
 import java.security.cert.X509Certificate;
 import java.text.SimpleDateFormat;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Collections;
-import java.util.List;
-import java.util.UUID;
+import java.util.*;
 
 /**
  * Class that signs your XML.
@@ -185,14 +175,23 @@ public class MakeXmlSignature {
      * @param sap the XmlSignatureAppearance
      * @param externalSignature  the interface providing the actual signing
      * @param chain the certificate chain
+     * @param includeSignaturePolicy if true SignaturePolicyIdentifier will be included (XAdES-EPES)
      * @throws GeneralSecurityException
      * @throws IOException
      * @throws DocumentException
      */
-    public static void signXadesBes(XmlSignatureAppearance sap, ExternalSignature externalSignature, Certificate[] chain)
+    public static void signXades(XmlSignatureAppearance sap, ExternalSignature externalSignature, Certificate[] chain,
+                                 boolean includeSignaturePolicy)
             throws GeneralSecurityException, DocumentException, IOException {
 
         verifyArguments(sap, externalSignature);
+
+        String signatureMethod = null;
+        if (externalSignature.getEncryptionAlgorithm().equals(SecurityConstants.RSA))
+            signatureMethod = SignatureMethod.RSA_SHA1;
+        else if (externalSignature.getEncryptionAlgorithm().equals(SecurityConstants.DSA))
+            signatureMethod = SignatureMethod.DSA_SHA1;
+
         String contentReferenceId = SecurityConstants.Reference_ + getRandomId();
         String signedPropertiesId = SecurityConstants.SignedProperties_ + getRandomId();
         String signatureId = SecurityConstants.Signature_ + getRandomId();
@@ -200,18 +199,23 @@ public class MakeXmlSignature {
         XMLSignatureFactory fac = createSignatureFactory();
 
         KeyInfo keyInfo = generateKeyInfo(chain, sap);
-        XMLObject xmlObject = generateXadesBesObject(fac, sap, signatureId, contentReferenceId, signedPropertiesId);
+        String[] signaturePolicy = null;
+        if (includeSignaturePolicy) {
+            signaturePolicy = new String[2];
+            if (signatureMethod.equals(SignatureMethod.RSA_SHA1)) {
+                signaturePolicy[0] = SecurityConstants.OID_RSA_SHA1;
+                signaturePolicy[1] = SecurityConstants.OID_RSA_SHA1_DESC;
+            } else {
+                signaturePolicy[0] = SecurityConstants.OID_DSA_SHA1;
+                signaturePolicy[1] = SecurityConstants.OID_DSA_SHA1_DESC;
+            }
+        }
+        XMLObject xmlObject = generateXadesObject(fac, sap, signatureId, contentReferenceId, signedPropertiesId, signaturePolicy);
         Reference contentReference = generateContentReference(fac, sap, contentReferenceId);
         Reference signedPropertiesReference = generateCustomReference(fac, "#"+signedPropertiesId, SecurityConstants.SignedProperties_Type, null);
 
         List<Reference> references = Arrays.asList(signedPropertiesReference, contentReference);
 
-        String signatureMethod = null;
-        if (externalSignature.getEncryptionAlgorithm().equals(SecurityConstants.RSA))
-            signatureMethod = SignatureMethod.RSA_SHA1;
-        else if (externalSignature.getEncryptionAlgorithm().equals(SecurityConstants.DSA))
-            signatureMethod = SignatureMethod.DSA_SHA1;
-
         // Create the SignedInfo
         DOMSignedInfo signedInfo = (DOMSignedInfo)fac.newSignedInfo(
                 fac.newCanonicalizationMethod(CanonicalizationMethod.INCLUSIVE, (C14NMethodParameterSpec) null),
@@ -222,6 +226,34 @@ public class MakeXmlSignature {
         sap.close();
     }
 
+    /**
+     * Signs the xml with XAdES BES using the enveloped mode, with optional xpath transform (see XmlSignatureAppearance).
+     * @param sap the XmlSignatureAppearance
+     * @param externalSignature  the interface providing the actual signing
+     * @param chain the certificate chain
+     * @throws GeneralSecurityException
+     * @throws IOException
+     * @throws DocumentException
+     */
+    public static void signXadesBes(XmlSignatureAppearance sap, ExternalSignature externalSignature, Certificate[] chain)
+            throws GeneralSecurityException, DocumentException, IOException {
+        signXades(sap, externalSignature, chain, false);
+    }
+
+    /**
+     * Signs the xml with XAdES BES using the enveloped mode, with optional xpath transform (see XmlSignatureAppearance).
+     * @param sap the XmlSignatureAppearance
+     * @param externalSignature  the interface providing the actual signing
+     * @param chain the certificate chain
+     * @throws GeneralSecurityException
+     * @throws IOException
+     * @throws DocumentException
+     */
+    public static void signXadesEpes(XmlSignatureAppearance sap, ExternalSignature externalSignature, Certificate[] chain)
+            throws GeneralSecurityException, DocumentException, IOException {
+        signXades(sap, externalSignature, chain, true);
+    }
+
     private static void verifyArguments(XmlSignatureAppearance sap, ExternalSignature externalSignature)
             throws DocumentException {
         if (sap.getXmlLocator() == null)
@@ -269,29 +301,30 @@ public class MakeXmlSignature {
         return XMLSignatureFactory.getInstance("DOM", new org.apache.jcp.xml.dsig.internal.dom.XMLDSigRI());
     }
 
-    private static XMLObject generateXadesBesObject(XMLSignatureFactory fac, XmlSignatureAppearance sap,
-        String signatureId, String contentReferenceId, String signedPropertiesId) throws GeneralSecurityException {
+    private static XMLObject generateXadesObject(XMLSignatureFactory fac, XmlSignatureAppearance sap,
+            String signatureId, String contentReferenceId, String signedPropertiesId, String[] signaturePolicy)
+            throws GeneralSecurityException {
 
         MessageDigest md = MessageDigest.getInstance(SecurityConstants.SHA1);
         Certificate cert = sap.getCertificate();
 
         org.w3c.dom.Document doc = sap.getXmlLocator().getDocument();
 
-        Element QualifyingProperties = doc.createElement(SecurityConstants.XADES_QualifyingProperties);
+        Element QualifyingProperties = doc.createElementNS(SecurityConstants.XADES_132_URI, SecurityConstants.XADES_QualifyingProperties);
         QualifyingProperties.setAttribute("Target", "#"+signatureId);
-            Element SignedProperties = doc.createElement(SecurityConstants.XADES_SignedProperties);
+            Element SignedProperties = doc.createElementNS(SecurityConstants.XADES_132_URI, SecurityConstants.XADES_SignedProperties);
             SignedProperties.setAttribute("Id", signedPropertiesId);
             SignedProperties.setIdAttribute("Id", true);
-                Element SignedSignatureProperties = doc.createElement(SecurityConstants.XADES_SignedSignatureProperties);
-                    Element SigningTime = doc.createElement(SecurityConstants.XADES_SigningTime);
+                Element SignedSignatureProperties = doc.createElementNS(SecurityConstants.XADES_132_URI, SecurityConstants.XADES_SignedSignatureProperties);
+                    Element SigningTime = doc.createElementNS(SecurityConstants.XADES_132_URI, SecurityConstants.XADES_SigningTime);
                         SimpleDateFormat sdf = new SimpleDateFormat(SecurityConstants.SigningTimeFormat);
                         String result = sdf.format(sap.getSignDate().getTime());
                         result = result.substring(0, result.length()-2).concat(":").concat(result.substring(result.length()-2));
                     SigningTime.appendChild(doc.createTextNode(result));
                 SignedSignatureProperties.appendChild(SigningTime);
-                    Element SigningCertificate = doc.createElement(SecurityConstants.XADES_SigningCertificate);
-                        Element Cert = doc.createElement(SecurityConstants.XADES_Cert);
-                            Element CertDigest = doc.createElement(SecurityConstants.XADES_CertDigest);
+                    Element SigningCertificate = doc.createElementNS(SecurityConstants.XADES_132_URI, SecurityConstants.XADES_SigningCertificate);
+                        Element Cert = doc.createElementNS(SecurityConstants.XADES_132_URI, SecurityConstants.XADES_Cert);
+                            Element CertDigest = doc.createElementNS(SecurityConstants.XADES_132_URI, SecurityConstants.XADES_CertDigest);
                                 Element DigestMethod = doc.createElementNS(SecurityConstants.XMLDSIG_URI, SecurityConstants.DigestMethod);
                                 DigestMethod.setAttribute(SecurityConstants.Algorithm, SecurityConstants.SHA1_URI);
                             CertDigest.appendChild(DigestMethod);
@@ -300,7 +333,7 @@ public class MakeXmlSignature {
                             CertDigest.appendChild(DigestValue);
                         Cert.appendChild(CertDigest);
                         if (cert instanceof X509Certificate) {
-                            Element IssueSerial = doc.createElement(SecurityConstants.XADES_IssuerSerial);
+                            Element IssueSerial = doc.createElementNS(SecurityConstants.XADES_132_URI, SecurityConstants.XADES_IssuerSerial);
                                 Element X509IssuerName = doc.createElementNS(SecurityConstants.XMLDSIG_URI, SecurityConstants.X509IssuerName);
                                 X509IssuerName.appendChild(doc.createTextNode(getX509IssuerName((X509Certificate)cert)));
                             IssueSerial.appendChild(X509IssuerName);
@@ -311,6 +344,31 @@ public class MakeXmlSignature {
                         }
                     SigningCertificate.appendChild(Cert);
                 SignedSignatureProperties.appendChild(SigningCertificate);
+                if (signaturePolicy != null) {
+                    Element SignaturePolicyIdentifier = doc.createElementNS(SecurityConstants.XADES_132_URI, SecurityConstants.XADES_SignaturePolicyIdentifier);
+                        Element SignaturePolicyId = doc.createElementNS(SecurityConstants.XADES_132_URI, SecurityConstants.XADES_SignaturePolicyId);
+                            Element SigPolicyId = doc.createElementNS(SecurityConstants.XADES_132_URI, SecurityConstants.XADES_SigPolicyId);
+                                Element Identifier = doc.createElementNS(SecurityConstants.XADES_132_URI, SecurityConstants.XADES_Identifier);
+                                Identifier.appendChild(doc.createTextNode(signaturePolicy[0]));
+                                Identifier.setAttribute(SecurityConstants.Qualifier, SecurityConstants.OIDAsURN);
+                            SigPolicyId.appendChild(Identifier);
+                    //ANSI X9.57 DSA signature generated with SHA-1 hash (DSA x9.30)
+                                Element Description = doc.createElementNS(SecurityConstants.XADES_132_URI, SecurityConstants.XADES_Description);
+                                Description.appendChild(doc.createTextNode(signaturePolicy[1]));
+                            SigPolicyId.appendChild(Description);
+                        SignaturePolicyId.appendChild(SigPolicyId);
+                            Element SigPolicyHash = doc.createElementNS(SecurityConstants.XADES_132_URI, SecurityConstants.XADES_SigPolicyHash);
+                                DigestMethod = doc.createElementNS(SecurityConstants.XMLDSIG_URI, SecurityConstants.DigestMethod);
+                                DigestMethod.setAttribute(SecurityConstants.Algorithm, SecurityConstants.SHA1_URI);
+                            SigPolicyHash.appendChild(DigestMethod);
+                                DigestValue = doc.createElementNS(SecurityConstants.XMLDSIG_URI, SecurityConstants.DigestValue);
+                                byte[] policyIdContent = getByteArrayOfNode(SigPolicyId);
+                                DigestValue.appendChild(doc.createTextNode(Base64.encode(md.digest(policyIdContent))));
+                            SigPolicyHash.appendChild(DigestValue);
+                        SignaturePolicyId.appendChild(SigPolicyHash);
+                    SignaturePolicyIdentifier.appendChild(SignaturePolicyId);
+                SignedSignatureProperties.appendChild(SignaturePolicyIdentifier);
+                }
                 SignedProperties.appendChild(SignedSignatureProperties);
                 Element SignedDataObjectProperties = doc.createElement(SecurityConstants.XADES_SignedDataObjectProperties);
                     Element DataObjectFormat = doc.createElement(SecurityConstants.XADES_DataObjectFormat);
@@ -403,4 +461,17 @@ public class MakeXmlSignature {
             throw new DocumentException(e);
         }
     }
+
+    private static byte[] getByteArrayOfNode(Node node) {
+        ByteArrayOutputStream stream = new ByteArrayOutputStream();
+        try {
+            StreamResult xmlOutput = new StreamResult(new StringWriter());
+            Transformer transformer = TransformerFactory.newInstance().newTransformer();
+            transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
+            transformer.transform(new DOMSource(node), xmlOutput);
+            return xmlOutput.getWriter().toString().getBytes();
+        } catch (Exception e) {
+        }
+        return stream.toByteArray();
+    }
 }
